@@ -34,26 +34,34 @@ class PageFactory(object):
     def __getattr__(self, loc):
 
         if loc in self.locators.keys():
-            self.locators[loc] = list(self.locators[loc])
-            self.locators[loc][0] = self.TYPE_OF_LOCATORS[self.locators[loc][0].lower()]
-            self.locators[loc] = tuple(self.locators[loc])
+            locator = (self.TYPE_OF_LOCATORS[self.locators[loc][0].lower()], self.locators[loc][1])
             try:
                 element = WebDriverWait(self.driver, self.timeout).until(
-                    EC.presence_of_element_located(self.locators[loc])
+                    EC.presence_of_element_located(locator)
                 )
             except (StaleElementReferenceException, NoSuchElementException, TimeoutException) as e:
-                raise Exception("An exception of type " + type(e).__name__ + " occurred. With Element -: " + loc)
+                raise Exception(
+                    "An exception of type " + type(e).__name__ +
+                    " occurred. With Element -: " + loc +
+                    " - locator: (" + locator[0] + ", " + locator[1] + ")"
+                )
 
             try:
                 element = WebDriverWait(self.driver, self.timeout).until(
-                    EC.visibility_of_element_located(self.locators[loc])
+                    EC.visibility_of_element_located(locator)
                 )
             except (StaleElementReferenceException, NoSuchElementException, TimeoutException) as e:
-                raise Exception("An exception of type " + type(e).__name__ + " occurred. With Element -: " + loc)
+                raise Exception(
+                    "An exception of type " + type(e).__name__ +
+                    " occurred. With Element -: " + loc +
+                    " - locator: (" + locator[0] + ", " + locator[1] + ")"
+                )
 
-            element = self.get_web_element(*self.locators[loc])
-            element._locator = self.locators[loc]
+            element = self.get_web_element(*locator)
+            element._locator = locator
             return element
+        else:
+            return super().__getattr__(loc)
 
     def get_web_element(self, *loc):
         element = self.driver.find_element(*loc)
@@ -61,26 +69,56 @@ class PageFactory(object):
         return element
 
     def highlight_web_element(self, element):
+        """
+        To highlight webElement
+        :param: WebElement
+        :return: None
+        """
         if self.highlight:
             self.driver.execute_script("arguments[0].style.border='2px ridge #33ffff'", element)
 
     def select_element_by_text(self, text):
+        """
+        Select webElement from dropdown list
+        :param: Text of Item in dropdown
+        :return: None
+        """
         select = Select(self)
         select.select_by_visible_text(text)
 
     def select_element_by_index(self, index):
+        """
+        Select webElement from dropdown list
+        :param: Index of Item in dropdown
+        :return: None
+        """
         select = Select(self)
         select.select_by_index(index)
 
     def select_element_by_value(self, value):
+        """
+        Select webElement from dropdown list
+        :param: value of Item in dropdown
+        :return: None
+        """
         select = Select(self)
         select.select_by_value(value)
 
     def get_list_item_count(self):
+        """
+        Count of Item from Dropdown
+        :param: None
+        :return: count
+        """
         select = Select(self)
         return len(select.options)
 
     def get_all_list_item(self):
+        """
+        Get list of Item from Dropdown
+        :param: None
+        :return: list of items present in dropdown
+        """
         select = Select(self)
         list_item = []
         for item in select.options:
@@ -88,6 +126,11 @@ class PageFactory(object):
         return list_item
 
     def get_list_selected_item(self):
+        """
+        Get list of Selected item in Dropdown
+        :param: None
+        :return: list of items selected in dropdown
+        """
         select = Select(self)
         list_item = []
         for item in select.all_selected_options:
@@ -95,36 +138,91 @@ class PageFactory(object):
         return list_item
 
     def click_button(self):
+        """
+        Perform  click on webElement
+        :param: None
+        :return: webElement
+        """
         self.element_to_be_clickable()
         self.click()
         return self
 
     def double_click(self):
+        """
+        perform Double click on webElement
+        :param: None
+        :return: webElement
+        """
         self.element_to_be_clickable()
         ActionChains(self.parent).double_click(self).perform()
         return self
 
+    def context_click(self):
+        """
+        perform Right click on webElement
+        :param: None
+        :return: webElement
+        """
+        self.element_to_be_clickable()
+        ActionChains(self.parent).context_click(self).perform()
+        return self
+
     def set_text(self, value):
+        """
+        type text in input box
+        :param: Text to be Enter
+        :return: webElement
+        """
         self.element_to_be_clickable()
         self.send_keys(value)
         return self
 
     def get_text(self):
+        """
+        get text from input box
+        :param: None
+        :return: text from webElement
+        """
         return self.text
 
     def clear_text(self):
+        """
+        Clear text from EditBox
+        :param: None
+        :return: None
+        """
         self.clear()
 
     def hover(self):
+        """
+        perform hover operation on webElement
+        :param: None
+        :return: None
+        """
         ActionChains(self.parent).move_to_element(self).perform()
 
     def is_Checked(self):
+        """
+        Check Radio button / CheckBox is selected
+        :param: None
+        :return: Boolean
+        """
         return self.isSelected()
 
     def is_Enabled(self):
+        """
+        get Enable state of webElement
+        :param: None
+        :return: Boolean
+        """
         return self.isEnabled()
 
     def getAttribute(self, attributeName):
+        """
+        get webElement attribute
+        :param: name of Attribute
+        :return: webElement attribute value
+        """
         return self.get_attribute(attributeName)
 
     def w3c(self):
@@ -140,6 +238,28 @@ class PageFactory(object):
             EC.element_to_be_clickable(self._locator)
         )
 
+    def text_to_be_present_in_element(self,text,timeout=None):
+        """
+        Wait till the text to be present in element
+        :param: text to be verified with element
+        """
+        if timeout is None:
+            timeout = PageFactory().timeout
+        return WebDriverWait(self.parent, timeout).until(
+            EC.text_to_be_present_in_element(self._locator,text_=text)
+        )
+
+    def element_located_selection_state_to_be(self,state,timeout=None):
+        """
+        Wait till element to be located and check if the selection state specified is in that state
+        :param: state to verified with an element in boolean
+        """
+        if timeout is None:
+            timeout = PageFactory().timeout
+        return WebDriverWait(self.parent, timeout).until(
+            EC.element_located_selection_state_to_be(self._locator,is_selected=state)
+        )
+
     def invisibility_of_element_located(self, timeout=None):
         """
         Wait till the element to be invisible
@@ -161,11 +281,17 @@ class PageFactory(object):
         )
 
     def execute_script(self, script):
-        self.parent.execute_script(script, self)
+        """
+        Execute JavaScript using web driver on selected web element
+        :param: Javascript to be execute
+        :return: None / depends on Script
+        """
+        return self.parent.execute_script(script, self)
 
 
 WebElement.click_button = PageFactory.click_button
 WebElement.double_click = PageFactory.double_click
+WebElement.context_click = PageFactory.context_click
 WebElement.element_to_be_clickable = PageFactory.element_to_be_clickable
 WebElement.invisibility_of_element_located = PageFactory.invisibility_of_element_located
 WebElement.visibility_of_element_located = PageFactory.visibility_of_element_located
@@ -184,3 +310,5 @@ WebElement.get_list_item_count = PageFactory.get_list_item_count
 WebElement.get_all_list_item = PageFactory.get_all_list_item
 WebElement.get_list_selected_item = PageFactory.get_list_selected_item
 WebElement.execute_script = PageFactory.execute_script
+WebElement.text_to_be_present_in_element = PageFactory.text_to_be_present_in_element
+WebElement.element_located_selection_state_to_be = PageFactory.element_located_selection_state_to_be
